
#include "vtkIntegrateVariables.h"

#include "vtkCell.h"
#include "vtkCellData.h"
#include "vtkCellDataToPointData.h"
#include "vtkDataArray.h"
#include "vtkDataSet.h"
#include "vtkDataSetSurfaceFilter.h"
#include "vtkFloatArray.h"
#include "vtkObjectFactory.h"
#include "vtkPointDataToCellData.h"
#include "vtkPointData.h"
#include "vtkPoints.h"
#include "vtkPolyDataNormals.h"
#include "vtkPolyData.h"
#include "vtkTriangle.h"
#include "vtkTriangleFilter.h"

#include "vtkInformation.h"
#include "vtkInformationVector.h"

#include <vtkstd/string>
#include <vtksys/ios/sstream>

//-----------------------------------------------------------------------------
vtkCxxRevisionMacro(vtkIntegrateVariables, "$Revision: 1.8 $");
vtkStandardNewMacro(vtkIntegrateVariables);

//-----------------------------------------------------------------------------
vtkIntegrateVariables::vtkIntegrateVariables()
{
  this->AttributeMode = VTK_ATTRIBUTE_MODE_DEFAULT;
  this->WriteResult = 0;
  this->IntegratedValue = 0.0;
  this->OutputDirectory = NULL;

  // by default process active point scalars
  this->SetInputArrayToProcess(0,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS_THEN_CELLS,
                               vtkDataSetAttributes::SCALARS);
}
//-----------------------------------------------------------------------------
vtkIntegrateVariables::~vtkIntegrateVariables()
{
}
//---------------------------------------------------------------------------
double vtkIntegrateVariables::GetIntegratedValue()
{
  this->Update();
  return this->IntegratedValue;
}
//---------------------------------------------------------------------------
int vtkIntegrateVariables::RequestData(
  vtkInformation *vtkNotUsed(request),
  vtkInformationVector **inputVector,
  vtkInformationVector *outputVector)
{
   // get the info objects
  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
  vtkInformation *outInfo = outputVector->GetInformationObject(0);

  // get the input and ouptut
  vtkPolyData *input = vtkPolyData::SafeDownCast(
    inInfo->Get(vtkDataObject::DATA_OBJECT()));
  vtkPolyData *output = vtkPolyData::SafeDownCast(
    outInfo->Get(vtkDataObject::DATA_OBJECT()));

  int cellId;
  vtkCellData *outputCD = output->GetCellData();
  vtkPointData *outputPD = output->GetPointData();
  vtkDataArray *inputDA = NULL;
  vtkDataArray *da; double range[2];
  vtkDataArray *da1;
  inputDA = this->GetInputArrayToProcess(0,inputVector);

  vtkFloatArray *integvariable;
  vtkDataArray *pd;
  double p0[3],p1[3],p2[3];
  double normal[3], coords[3];
  double area=0.0, value=0.0, local=0.0;
  int attributeDataType, numCells;
  this->IntegratedValue = 0.0;

  vtkPointDataToCellData *p2c = vtkPointDataToCellData::New();
  vtkCellDataToPointData *c2p = vtkCellDataToPointData::New();

  if (!input)
    {
    vtkErrorMacro(<<"No input");
    return 0;
    }
  if(!inputDA)
    {
    vtkErrorMacro(<<"No Variable was found");
    return 1;
    }
  this->UpdateProgress(0.1);

  vtkTriangleFilter *cutTriangle = vtkTriangleFilter::New();
  cutTriangle->SetInput(input);

  vtkDataSetSurfaceFilter *sf = vtkDataSetSurfaceFilter::New();
  sf->SetInput(cutTriangle->GetOutput());
  sf->Update();
  vtkDebugMacro(<<"Outside surface extracted were\n");

  output->CopyStructure((vtkDataSet *)sf->GetOutput());

  vtkPolyDataNormals *normals = vtkPolyDataNormals::New();

  if (this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT)
    {
    outputPD->PassData(sf->GetOutput()->GetPointData());

    attributeDataType = 0;
    p2c->SetInput((vtkDataSet *)sf->GetOutput());
    p2c->PassPointDataOn();
    p2c->Update();
    numCells = p2c->GetPolyDataOutput()->GetNumberOfCells();

    normals->SetInput((vtkDataSet *)p2c->GetPolyDataOutput());
    normals->ComputeCellNormalsOn();
    normals->Update();
    vtkDebugMacro(<<"Normals were evaluated\n");

    da1 = p2c->GetPolyDataOutput()->GetCellData()->GetArray(inputDA->GetName());
  }
  else if (this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_POINT_DATA)
    {
    outputPD->PassData(sf->GetOutput()->GetPointData());

    attributeDataType = 0;
    p2c->SetInput((vtkDataSet *)sf->GetOutput());
    p2c->PassPointDataOn();
    p2c->Update();
    numCells = p2c->GetPolyDataOutput()->GetNumberOfCells();

    normals->SetInput((vtkDataSet *)p2c->GetPolyDataOutput());
    normals->ComputeCellNormalsOn();
    normals->Update();
    vtkDebugMacro(<<"Normals were evaluated\n");

    da1 = p2c->GetPolyDataOutput()->GetCellData()->GetArray(inputDA->GetName());
  }
  else
    {
    outputCD->PassData(sf->GetOutput()->GetCellData());

    attributeDataType = 1;
    numCells = sf->GetOutput()->GetNumberOfCells();

    normals->SetInput((vtkDataSet *)sf->GetOutput());
    normals->ComputeCellNormalsOn();
    normals->Update();
    vtkDebugMacro(<<"Normals were evaluated\n");

    da1 = sf->GetOutput()->GetCellData()->GetArray(inputDA->GetName());
 }


//   numCells = p2c->GetPolyDataOutput()->GetNumberOfCells();
  this->UpdateProgress(0.2);

  vtksys_ios::ostringstream str;
  str << "Locally integrated " << inputDA->GetName();

  integvariable = vtkFloatArray::New();
  integvariable->SetNumberOfComponents(1);
  integvariable->SetName(str.str().c_str());
  integvariable->SetNumberOfTuples(numCells);

  vtkCell *cell;
  vtkPoints *pt;

  vtkDebugMacro(<< "Integrating " << inputDA->GetName() << "\n");
  for(cellId=0; cellId < numCells; cellId++)
    {
    if ( ! (cellId % 100) )
      {
      this->UpdateProgress (0.2 + 0.8*(double)cellId/numCells);
      }
    cell = cutTriangle->GetOutput()->GetCell(cellId);
    pt = cell->GetPoints();

    pt->GetPoint(0, p0);
    pt->GetPoint(1, p1);
    pt->GetPoint(2, p2);

    area = vtkTriangle::TriangleArea(p0, p1, p2);

    da1->GetTuple(cellId, &value);
    local = area*value;

    this->IntegratedValue += local;
    integvariable->SetTuple1(cellId, local);
    }

  if (attributeDataType == 0)
    {
    p2c->GetPolyDataOutput()->GetCellData()->AddArray(integvariable);
    c2p->SetInput((vtkDataSet *)p2c->GetPolyDataOutput());
    c2p->Update();

    da = c2p->GetPolyDataOutput()->GetPointData()->GetArray(inputDA->GetName());
    da->GetRange(range,0);
    vtkDebugMacro(<< "Input point array" << inputDA->GetName() << "  ranges from :" << range [0] << " to "<< range[1] << "\n");

    pd = c2p->GetPolyDataOutput()->GetPointData()->GetArray(str.str().c_str());

    int arrayIdx = outputPD->AddArray(pd);
    outputPD->SetActiveAttribute(arrayIdx, vtkDataSetAttributes::SCALARS);
    }
  else
    {
    sf->GetOutput()->GetCellData()->AddArray(integvariable);

    da = sf->GetOutput()->GetPointData()->GetArray(inputDA->GetName());
    da->GetRange(range,0);
    vtkDebugMacro(<< "Input cell array" << inputDA->GetName() << "  ranges from :" << range [0] << " to "<< range[1] << "\n");

    pd = sf->GetOutput()->GetCellData()->GetArray(str.str().c_str());
    int arrayIdx = outputCD->AddArray(pd);
    outputCD->SetActiveAttribute(arrayIdx, vtkDataSetAttributes::SCALARS);
  }

  if (this->WriteResult)
    {
     vtksys_ios::ostringstream file;
     if (this->OutputDirectory == NULL) { file << "/tmp/Integrated_" << inputDA->GetName() << ".dat";}
     else {file << this->OutputDirectory << "/Integrated_" << inputDA->GetName() << ".dat";}

     vtksys_ios::ostringstream result;
     vtkDebugMacro(<< "Integrated " << inputDA->GetName() << " value = "  << this->IntegratedValue << "\n");
     result << this->IntegratedValue << "\n";

     const char *filename = file.str().c_str();
     ofstream f;
     // appens integration values in the same file (for time-dependent data)
     f.open( filename,ios::app );
     f.write(result.str().c_str(),strlen(result.str().c_str()));
     f.close();
    }
  vtkDebugMacro(<< "Results have been written to file\n");


  normals->Delete();
  sf->Delete();
  this->UpdateProgress(1.0);
  return 1;
}

const char* vtkIntegrateVariables::GetAttributeModeAsString()
{
  if ( this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT )
    {
    return "Default";
    }
  else if ( this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_POINT_DATA )
    {
    return "UsePointData";
    }
  else 
    {
    return "UseCellData";
    }
}

//-----------------------------------------------------------------------------
void vtkIntegrateVariables::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}
