// .NAME vtkIntegrateVariables -- Integrate a variable over a surface
// .SECTION Description
// vtkIntegrateVariables is a polydata filter object 
 
#ifndef __vtkIntegrateVariables_h
#define __vtkIntegrateVariables_h

#include "vtkPolyDataAlgorithm.h"

#define VTK_ATTRIBUTE_MODE_DEFAULT 0
#define VTK_ATTRIBUTE_MODE_USE_POINT_DATA 1
#define VTK_ATTRIBUTE_MODE_USE_CELL_DATA 2

class VTK_EXPORT vtkIntegrateVariables : public vtkPolyDataAlgorithm
{
public:

  vtkTypeRevisionMacro(vtkIntegrateVariables, vtkPolyDataAlgorithm);

  void PrintSelf(ostream& os, vtkIndent indent);

  static vtkIntegrateVariables *New();

  // Description:
  // Control whether the filter operates on point data or cell data.
  // By default (AttributeModeToDefault), the filter uses point
  // data. Alternatively you can explicitly set the filter to use point data
  // (AttributeModeToUsePointData) or cell data (AttributeModeToUseCellData).
  vtkSetMacro(AttributeMode,int);
  vtkGetMacro(AttributeMode,int);
  void SetAttributeModeToDefault() 
    {this->SetAttributeMode(VTK_ATTRIBUTE_MODE_DEFAULT);};
  void SetAttributeModeToUsePointData() 
    {this->SetAttributeMode(VTK_ATTRIBUTE_MODE_USE_POINT_DATA);};
  void SetAttributeModeToUseCellData() 
    {this->SetAttributeMode(VTK_ATTRIBUTE_MODE_USE_CELL_DATA);};

  const char *GetAttributeModeAsString();  double GetIntegratedValue();

  vtkSetStringMacro(OutputDirectory);
  vtkGetStringMacro(OutputDirectory);

  vtkSetMacro(WriteResult,int);
  vtkGetMacro(WriteResult,int);
  vtkBooleanMacro(WriteResult,int);

 protected:
  vtkIntegrateVariables();
  ~vtkIntegrateVariables();

  virtual int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);

  int AttributeMode;
  int WriteResult;
  double IntegratedValue;
  char *OutputDirectory;

 private:
  vtkIntegrateVariables(const vtkIntegrateVariables&); // not implemented
  void operator=(const vtkIntegrateVariables&); // not implemented
};

#endif
